/******************************************************************************
 *                                                                            *
 * LISTSERV V2 - List file viewer (unix/NT/VMS)                               *
 *                                                                            *
 *       Copyright L-Soft international 1996-98 - All rights reserved         *
 *                                                                            *
 * This utility can be used to view files with an extension of LIST.          *
 *                                                                            *
 ******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#define LRECL		100
#define SHORT_LRECL	80
#define INIT_ENTRIES	50000
#define ADD_ENTRIES	10000

#ifdef vms
#define LISTDIR		"LISTSERV_LISTS_DIR:"
#else
#define LISTDIR
#endif

int main(argc, argv)
int argc;
char **argv;
{
	FILE *f, *fo;
	unsigned char buf[LRECL + 100], *files[1024], **entries, *r, c;
	char filename[256], *fileid, *fn;
	int mult, n, p, s_opt, h_opt, a_opt, e_opt, eh_opt, cfile, nfiles,
	    count, rndsel, nentries, maxentries, limit;

	mult = s_opt = h_opt = a_opt = e_opt = eh_opt = 0;
	rndsel = 0;
	for (nfiles = 0, argc--, argv++; argc-- > 0;) {
		fileid = *argv++;
		if (fileid[0] == '-'
#if defined(_WIN32) || defined(vms)
				     || fileid[0] == '/'
#endif
							) {
			for (; c = *++fileid;) {
				switch (c) {
					case 'e':
					case 'E':
						e_opt = 1;
						if (*(fileid+1) == 'h') {
							fileid++;
							eh_opt = 1;
						}
					case 's':
					case 'S':
						s_opt = 1;
						break;

					case 'h':
					case 'H':
						h_opt = 1;
						break;

					case 'a':
					case 'A':
						a_opt = 1;
						break;

					case 'r':
					case 'R':
						if (argc > 0) {
							argc--;
							rndsel = atoi(*argv++);
							if (rndsel < 0)
								rndsel = 0;
						}
						break;

					default:
						goto Syntax;
				}
			}
			continue;
		}
		files[nfiles++] = (unsigned char *)fileid;
	}

	if (!nfiles)
		goto Syntax;
	if (nfiles > 1)
		mult = 1;
	if (rndsel) {
		a_opt = 0;
		nentries = 0;
		maxentries = INIT_ENTRIES;
		entries = (unsigned char **)malloc(INIT_ENTRIES * sizeof(unsigned char *));
		srand(time(0));
	}

	for (cfile = 0; cfile < nfiles;) {
		fileid = (char *)files[cfile++];
		if (!strrchr(fileid, '.')) {
			sprintf(filename, LISTDIR "%s.list", fileid);
			fileid = filename;
		}
		if (!(f = fopen(fileid, "rb"))) {
			printf("%s: not found.\n", fileid);
			continue;
		}

#ifndef vms
		if (fread(buf, 1, 11, f) != 11 ||
		    strncmp(buf, "*LSV-F*", 7) ||
		    buf[7] || buf[8] || buf[9] ||
		    buf[10] != LRECL) {
			printf("%s: not a LISTSERV list file.\n", fileid);
			fclose(f);
			continue;
		}
#endif

		if (mult)
			printf("File %s:\n", fileid);

		for (;;) {
			n = fread(buf, 1, LRECL, f);
			if (n != LRECL) {
				if (!n)
					break;
				printf("\n\
>>> Incomplete last record, %d bytes missing.\n\
-> Incomplete last record follows:\n\n", LRECL - n);
				for (r = buf; n--;)
					putchar(*r++);
				putchar('\n');
				break;
			}
			if (buf[0] == ' ' && !a_opt)
				continue;

			if (buf[0] == '*') {
				if (e_opt || rndsel)
					continue;
			}
			else if (h_opt)
				break;
			else if (s_opt)
				n = SHORT_LRECL;
			for (; n && buf[n-1] == ' '; n--);
			buf[n] = '\0';
			if (e_opt) {
				for (r = buf; *r; r++)
					if (*r == ' ') {
						*r = '\0';
						break;
					}
				if (eh_opt) {
					for (r = buf; *r && *r != '@'; r++);
					if (*r)
						strcpy(buf, ++r);
				}
			}
			if (rndsel) {
				if (nentries >= maxentries) {
					maxentries += ADD_ENTRIES;
					entries = (unsigned char **)realloc(entries, maxentries * sizeof(unsigned char *));
				}
				r = (unsigned char *)malloc(strlen((char *)buf) + 1);
				strcpy(entries[nentries++] = r, buf);
				continue;
			}
			printf("%s\n", buf);
		}
		fclose(f);

		if (rndsel) {
			if (fileid != filename)
				strcpy(filename, fileid);
			for (fn = filename + strlen(filename); --fn > filename && *fn != '.';);
			if (*fn++ != '.') {
				fn = filename + strlen(filename);
				*fn++ = '.';
			}
			strcpy(fn, "view1");
			if (!(fo = fopen(filename, "w"))) {
				printf("Error opening '%s' for writing.\n", filename);
				continue;
			}
			for (n = 0; n < nentries; n++) {
				p = ((rand() & 0x7FFF) << 16) + (rand() & 0xFFFF);
				p %= nentries;
				r = entries[n];
				entries[n] = entries[p];
				entries[p] = r;
			}
			limit = rndsel;
			if (limit > nentries)
				limit = nentries;
			for (n = 0; n < limit;)
				fprintf(fo, "%s\n", entries[n++]);
			fclose(fo);
			strcpy(fn, "view2");
			if (!(fo = fopen(filename, "w"))) {
				printf("Error opening '%s' for writing.\n", filename);
				continue;
			}
			for (; n < nentries;)
				fprintf(fo, "%s\n", entries[n++]);
			fclose(fo);
		}

		if (mult)
			putchar('\n');
	}
	return(0);

Syntax:
	printf("Syntax: listview [-a] [-e[h]] [-h] [-r nnnn] [-s] file1 file2...\n");
	return(1);
}
